"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GraphqlOrmDataSource = void 0;
const bson_1 = require("bson");
const apollo_datasource_1 = require("apollo-datasource");
const apollo_server_caching_1 = require("apollo-server-caching");
class GraphqlOrmDataSource extends apollo_datasource_1.DataSource {
    constructor(ormConfig) {
        var _a, _b;
        super();
        this.orm = ormConfig.orm;
        this.useCache = !!ormConfig.cache;
        this.cacheTtl = (_b = (_a = ormConfig.cache) === null || _a === void 0 ? void 0 : _a.milliseconds) !== null && _b !== void 0 ? _b : 1000 * 30;
    }
    initialize(config) {
        this.context = config.context;
        if (this.useCache) {
            this.cache = config.cache || new apollo_server_caching_1.InMemoryLRUCache();
        }
    }
    em() {
        return this.orm.manager;
    }
    repo(target) {
        return this.orm.getRepository(target);
    }
    getEntityCacheKey(repoName, id, options = {}) {
        return bson_1.EJSON.stringify({ repoName, id, options });
    }
    cacheGet(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.cache) {
                const cachedData = yield this.cache.get(key);
                if (cachedData) {
                    const parsed = bson_1.EJSON.parse(cachedData);
                    if (parsed) {
                        return Promise.resolve(parsed);
                    }
                }
            }
            return Promise.resolve(undefined);
        });
    }
    cacheSet(key, val) {
        if (this.cache) {
            const cacheStr = bson_1.EJSON.stringify(val);
            this.cache.set(key, cacheStr, { ttl: this.cacheTtl });
        }
    }
    findOne(target, id, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const repo = this.repo(target);
            const repoName = typeof repo;
            const key = this.getEntityCacheKey(repoName, id, options);
            let data = yield this.cacheGet(key);
            if (!data) {
                data = yield repo.findOne(id, options);
                if (data)
                    this.cacheSet(key, data);
            }
            return Promise.resolve(data);
        });
    }
    findManyByIds(target, ids, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const repo = this.repo(target);
            const repoName = typeof repo;
            let data = [];
            if (this.cache) {
                let key;
                const idsNotFound = [];
                let row;
                for (let id of ids) {
                    key = this.getEntityCacheKey(repoName, id);
                    row = yield this.cacheGet(key);
                    if (row) {
                        data.push(row);
                    }
                    else {
                        idsNotFound.push(id);
                    }
                }
                for (let id of idsNotFound) {
                    row = yield this.findOne(target, id);
                    data.push(row);
                }
            }
            else {
                const rows = yield repo.findByIds(ids, options);
                data = rows;
            }
            return Promise.resolve(data);
        });
    }
    saveOne(entity, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.orm.manager.save(entity, options);
        });
    }
    removeOne(entity, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.orm.manager.remove(entity, options);
        });
    }
}
exports.GraphqlOrmDataSource = GraphqlOrmDataSource;
